@@ -1,10 +1,7 @@
-import os
-
 from OpFlowLab.GUI.pathWidget import pathWidget
 from PyQt5.QtCore import pyqtSignal, Qt, pyqtSlot
 from PyQt5.QtWidgets import QWidget, QGridLayout, QLineEdit, QLabel, \
     QVBoxLayout, QGroupBox, QPushButton
-from ruamel.yaml import YAML
 
 from .opFlowLabWorker import opFlowLabConfig
 from .styleSheet import invalid_stylesheet, valid_stylesheet
@@ -114,7 +111,7 @@ class advancedConfigTab(QWidget):
         self.config.flowWarp_folder = self.flowWarpFolderLineEdit.text()
         self.config.executablePath = self.executablePathWidget.getPath()
         self.config.saveSettings()
-        self.messageSignal.emit("Settings have been saved to {}".format(self.config.settingsPath), "info")
+        self.messageSignal.emit("Settings have been saved to \"{}\"".format(self.config.settingsPath), "info")
 
     @staticmethod
     def stringCheck(text):
