@@ -3,6 +3,7 @@ import time
 from datetime import datetime
 
 import numpy as np
+from skimage import util
 from PyQt5.QtCore import QObject, pyqtSignal, QCoreApplication
 from ruamel.yaml import YAML
 
@@ -41,11 +42,12 @@ class opFlowLabConfig(Singleton):
             self.max_object_size = None
             self.save_velocities_as_tif = False
 
-            self.flowPath_folder = "flowPath"
+            self.flowPath_folder = "FlowPath"
             self.start_frame = 0
             self.frames = 10
             self.spacing = 8
             self.max_tracers = None
+            self.metric = "Angle"
 
             self.flowTracer_folder = "FlowTracer"
             self.use_random_tracers = True
@@ -61,10 +63,11 @@ class opFlowLabConfig(Singleton):
 
             self.colormap = "None"
             self.smoothing = None
-            self.colormap_min = -1.5
-            self.colormap_max = 1.5
+            self.colorbarMin = None
+            self.colorbarMax = None
 
             self.images = []
+            self.colormap_vis = []
             self.log_file = ""
 
             self.action_count = 1
@@ -205,8 +208,7 @@ class opFlowLabWorker(QObject):
                 QCoreApplication.processEvents()
 
                 if self.__abort is True:
-                    self.finishSignal.emit()
-                    return
+                    break
                 else:
                     start = time.time()
                     self.progressSignal.emit(int(100 * img_no / (instance.length-1)))
@@ -237,7 +239,13 @@ class opFlowLabWorker(QObject):
         self.startSignal.emit()
         try:
             instance = self.config.getInstance()
-            self.config.images = []
+            self.config.images = np.zeros((instance.length-1-self.config.start_frame,) + instance.image_shape + (3,), dtype=np.uint8)
+            if self.config.metric == "Angle":
+                self.config.colormap_vis = np.zeros(
+                    (instance.length - 1 - self.config.start_frame,) + (800, 800) + (3,), dtype=np.uint8)
+            else:
+                self.config.colormap_vis = np.zeros(
+                    (instance.length - 1 - self.config.start_frame,) + (100, 600) + (3,), dtype=np.uint8)
 
             # create output folders
             flow_trace_folder = os.path.join(self.config.forwardFolderPath, self.config.flowPath_folder)
@@ -252,22 +260,28 @@ class opFlowLabWorker(QObject):
             for img_no in range(self.config.start_frame, instance.length - 1):
                 QCoreApplication.processEvents()
                 if self.__abort is True:
-                    self.finishSignal.emit()
-                    return
+                    break
                 else:
                     start = time.time()
                     self.progressSignal.emit(int(100 * (img_no - self.config.start_frame)
                                                  / (instance.length - self.config.start_frame - 1)))
+                    tracer_pos, output_image, colormap_vis = instance.flowPath_iteration(img_no, tracer_pos,
+                                                                                         all_pos_index, x_array_shape,
+                                                                                         flow_trace_folder,
+                                                                                         alpha=self.config.alpha,
+                                                                                         img_frame=instance.image[
+                                                                                             img_no],
+                                                                                         frames=self.config.frames,
+                                                                                         spacing=self.config.spacing,
+                                                                                         colormap=self.config.colormap,
+                                                                                         colormap_min=self.config.colorbarMin,
+                                                                                         colormap_max=self.config.colorbarMax,
+                                                                                         metric_type=self.config.metric,
+                                                                                         )
 
-                    tracer_pos, output_image = instance.flowPath_iteration(img_no, tracer_pos, all_pos_index, x_array_shape,
-                                                                           flow_trace_folder,
-                                                                           alpha=self.config.alpha,
-                                                                           img_frame=instance.image[img_no],
-                                                                           frames=self.config.frames,
-                                                                           spacing=self.config.spacing,
-                                                                           )
+                    self.config.images[img_no + self.config.start_frame, ...] = util.img_as_ubyte(output_image)
+                    self.config.colormap_vis[img_no + self.config.start_frame, ...] = colormap_vis
 
-                    self.config.images.append(output_image)
                     self.updateSignal.emit()
                     time_list = np.append(time_list, time.time()-start)
                     self.timeSignal.emit(np.mean(time_list)*(instance.length - 1 - (img_no + 1)))
@@ -282,7 +296,8 @@ class opFlowLabWorker(QObject):
         self.startSignal.emit()
         try:
             instance = self.config.getInstance()
-            self.config.images = []
+            self.config.images = np.zeros((instance.length-1-self.config.start_frame,) + instance.image_shape + (3,),
+                                          dtype=np.uint8)
 
             output_folder = os.path.join(self.config.forwardFolderPath, self.config.flowTracer_folder)
             os.makedirs(output_folder, exist_ok=True)
@@ -299,8 +314,7 @@ class opFlowLabWorker(QObject):
             for img_no in range(self.config.start_frame, instance.length - 1):
                 QCoreApplication.processEvents()
                 if self.__abort is True:
-                    self.finishSignal.emit()
-                    return
+                    break
                 else:
                     start = time.time()
                     self.progressSignal.emit(int(100 * (img_no - self.config.start_frame)
@@ -312,14 +326,14 @@ class opFlowLabWorker(QObject):
                                                                        self.config.start_frame,
                                                                        tracer_centroids, color,
                                                                        )
-                        self.config.images.append(output_image)
+                        self.config.images[0, ...] = output_image
 
                     tracer_centroids, output_image = instance.flowTracer_iteration(output_folder,
                                                                                    img_no,
                                                                                    tracer_centroids, color,
                                                                                    )
 
-                    self.config.images.append(output_image)
+                    self.config.images[img_no + self.config.start_frame, ...] = util.img_as_ubyte(output_image)
                     self.updateSignal.emit()
                     time_list = np.append(time_list, time.time()-start)
                     self.timeSignal.emit(np.mean(time_list)*(instance.length - 1 - (img_no + 1)))
@@ -334,7 +348,8 @@ class opFlowLabWorker(QObject):
         self.startSignal.emit()
         try:
             instance = self.config.getInstance()
-            self.config.images = []
+            self.config.images = np.zeros((instance.length-1-self.config.start_frame,) + instance.image_shape + (3,),
+                                          dtype=np.uint8)
 
             output_folder = os.path.join(self.config.forwardFolderPath, self.config.flowWarp_folder)
             os.makedirs(output_folder, exist_ok=True)
@@ -344,8 +359,7 @@ class opFlowLabWorker(QObject):
             for img_no in range(self.config.start_frame, instance.length - 1):
                 QCoreApplication.processEvents()
                 if self.__abort is True:
-                    self.finishSignal.emit()
-                    return
+                    break
                 else:
                     start = time.time()
                     self.progressSignal.emit(int(100 * (img_no - self.config.start_frame)
@@ -355,9 +369,10 @@ class opFlowLabWorker(QObject):
                     print("Frame {}: Normalized root mean squared error = {}".format(img_no + 1, nrmse))
 
                     output_image = np.moveaxis(output_image, 0, -1)
-                    display_image = np.zeros(output_image.shape[:2] + (3,))
+                    display_image = np.zeros(output_image.shape[:2] + (3,), dtype=output_image.dtype)
                     display_image[..., :2] = output_image
-                    self.config.images.append(display_image)
+                    self.config.images[img_no + self.config.start_frame, ...] = util.img_as_ubyte(display_image)
+
                     self.updateSignal.emit()
                     time_list = np.append(time_list, time.time() - start)
                     self.timeSignal.emit(np.mean(time_list) * (instance.length - 1 - (img_no + 1)))
@@ -372,7 +387,13 @@ class opFlowLabWorker(QObject):
         self.startSignal.emit()
         try:
             instance = self.config.getInstance()
-            self.config.images = []
+            if self.config.colormap != "None":
+                self.config.images = np.zeros((instance.length-1-self.config.start_frame,) + instance.image_shape + (3,), dtype=np.uint8)
+                self.config.colormap_vis = np.zeros(
+                    (instance.length - 1 - self.config.start_frame,) + (100, 600) + (3,), dtype=np.uint8)
+            else:
+                self.config.images = np.zeros(
+                    (instance.length - 1 - self.config.start_frame,) + instance.image_shape, dtype=np.float32)
 
             # create output folders
             output_folder = os.path.join(self.config.forwardFolderPath, self.config.derivative)
@@ -384,23 +405,27 @@ class opFlowLabWorker(QObject):
             for img_no in range(self.config.start_frame, instance.length - 1):
                 QCoreApplication.processEvents()
                 if self.__abort is True:
-                    self.finishSignal.emit()
-                    return
+                    break
                 else:
                     start = time.time()
                     self.progressSignal.emit(int(100 * (img_no - self.config.start_frame)
                                                  / (instance.length - self.config.start_frame - 1)))
 
-                    output_image = instance.flowDerivative_iteration(output_folder, img_no,
-                                                                     self.config.derivative_matrix,
-                                                                     derivative_string,
-                                                                     smoothing=self.config.smoothing,
-                                                                     colormap_min=self.config.colormap_min,
-                                                                     colormap_max=self.config.colormap_max,
-                                                                     colormap=self.config.colormap,
-                                                                     )
+                    output_image, colormap_vis = instance.flowDerivative_iteration(output_folder, img_no,
+                                                                                   self.config.derivative_matrix,
+                                                                                   derivative_string,
+                                                                                   smoothing=self.config.smoothing,
+                                                                                   colormap_min=self.config.colorbarMin,
+                                                                                   colormap_max=self.config.colorbarMax,
+                                                                                   colormap=self.config.colormap,
+                                                                                   )
+
+                    if self.config.colormap != "None":
+                        self.config.images[img_no + self.config.start_frame, ...] = util.img_as_ubyte(output_image)
+                        self.config.colormap_vis[img_no + self.config.start_frame, ...] = colormap_vis
+                    else:
+                        self.config.images[img_no + self.config.start_frame, ...] = output_image
 
-                    self.config.images.append(output_image)
                     self.updateSignal.emit()
                     time_list = np.append(time_list, time.time()-start)
                     self.timeSignal.emit(np.mean(time_list)*(instance.length - 1 - (img_no + 1)))
