@@ -6,7 +6,7 @@ from PyQt5.QtCore import pyqtSignal
 from PyQt5.QtWidgets import QWidget, QGridLayout, QPushButton, QLabel, QVBoxLayout, QComboBox, QHBoxLayout
 
 from .opFlowLabWorker import opFlowLabConfig
-from OpFlowLab.GUI.opticalFlow.nvidiaOptions import nvidiaOFOptions
+from ..GUI.opticalFlow.nvidiaOptions import nvidiaOFOptions
 from .opticalFlow.broxOptions import broxOptions
 from .opticalFlow.dualOptions import dualOptions
 from .opticalFlow.farnebackOptions import farnebackOptions
@@ -160,10 +160,10 @@ class OFWorker(QObject):
     def __init__(self):
         super(QObject, self).__init__()
         self.command = None
-        self.__abort = False
+        self._abort = False
 
     def runProcess(self):
-        self.__abort = False
+        self._abort = False
         for cmd in self.command:
             self.messageSignal.emit("", "info")
             self.messageSignal.emit("Running command: {}".format(cmd), "info")
@@ -177,20 +177,20 @@ class OFWorker(QObject):
                 if output:
                     self.messageSignal.emit(output.strip().replace("\\\\", "/"), "info")
 
-                if self.__abort == True:
+                if self._abort == True:
                     process.kill()
                     break
 
             _, errors = process.communicate()
-            if errors is not '':
+            if errors != '':
                 self.messageSignal.emit(errors.strip(), "error")
                 break
 
-            if self.__abort == True:
+            if self._abort == True:
                 break
 
         self.finishSignal.emit()
         # return rc
 
     def terminateCalculation(self):
-        self.__abort = True
+        self._abort = True
