@@ -204,7 +204,7 @@ class OpFlowLab:
                             frame_no,
                             flowx_output_directory, flowy_output_directory,
                             pairwise_threshold_distance=10,
-                            min_object_size=None, max_object_size=None,
+                            min_object_size=10, max_object_size=None,
                             save_dtype="float16",
                             save_velocity_as_tif=False,
                             ):
@@ -243,6 +243,8 @@ class OpFlowLab:
         --------
         object_matching_function : wrapper function to perform object matching and interpolation on all frames
         """
+        import copy
+
         # obtain initial centroids
         initial_image = skimage.io.imread(self.object_segmentation_filelist[frame_no])
         if initial_image.shape != self.image_shape:
@@ -250,22 +252,21 @@ class OpFlowLab:
             initial_image = cv2.resize(initial_image, (self.image_shape[1], self.image_shape[0]),
                                        interpolation=cv2.INTER_NEAREST)
         initial_object_properties = skimage.measure.regionprops(initial_image)
-        initial_object_properties = np.array(initial_object_properties, dtype=object)
-        initial_object_area = np.array([x['Area'] for x in initial_object_properties])
-        area_mask = np.ones_like(initial_object_area)
-        if min_object_size is not None:
-            area_mask = np.logical_and(area_mask, initial_object_area > min_object_size)
-        if max_object_size is not None:
-            area_mask = np.logical_and(area_mask, initial_object_area < max_object_size)
-        initial_object_properties = initial_object_properties[area_mask]
 
-        initial_properties = List([(prop["label"], prop["local_centroid"], prop["centroid"], prop["orientation"],
-                                    prop["major_axis_length"], prop["coords"],
-                                    object_matching.calculate_contour(prop["image"]))
-                                   for prop in initial_object_properties])
+        initial_properties = List()
+        for prop in initial_object_properties:
+            if min_object_size is None:
+                min_object_size = 0
+            if max_object_size is None:
+                max_object_size = np.inf
+
+            if min_object_size < prop["area"] < max_object_size:
+                initial_properties.append((prop["label"], prop["local_centroid"], prop["centroid"], prop["orientation"],
+                                          prop["major_axis_length"], np.array(prop["coords"], order='C', copy=True),
+                                          object_matching.calculate_contour(prop["image"])))
 
         counter = 0
-        total_number = initial_object_properties.shape[0]
+        total_number = len(initial_properties)
         matched_flow_x = np.full(self.image_shape, np.nan)
         matched_flow_y = np.full(self.image_shape, np.nan)
         centroid_list = [x[2] for x in initial_properties]
@@ -277,20 +278,18 @@ class OpFlowLab:
                                       interpolation=cv2.INTER_NEAREST)
 
         target_object_properties = skimage.measure.regionprops(target_image)
-        target_object_properties = np.array(target_object_properties, dtype=object)
 
-        target_object_area = np.array([x['Area'] for x in target_object_properties])
-        area_mask = np.ones_like(target_object_area)
-        if min_object_size is not None:
-            area_mask = np.logical_and(area_mask, target_object_area > min_object_size)
-        if max_object_size is not None:
-            area_mask = np.logical_and(area_mask, target_object_area < max_object_size)
-        target_object_properties = target_object_properties[area_mask]
+        target_properties = List()
+        for prop in target_object_properties:
+            if min_object_size is None:
+                min_object_size = 0
+            if max_object_size is None:
+                max_object_size = np.inf
 
-        target_properties = List([(prop["label"], prop["local_centroid"], prop["centroid"], prop["orientation"],
-                                   prop["major_axis_length"], prop["coords"],
-                                   object_matching.calculate_contour(prop["image"]))
-                                  for prop in target_object_properties])
+            if min_object_size < prop["area"] < max_object_size:
+                target_properties.append((prop["label"], prop["local_centroid"], prop["centroid"], prop["orientation"],
+                                          prop["major_axis_length"], np.array(prop["coords"], order='C', copy=True),
+                                          object_matching.calculate_contour(prop["image"])))
 
         # load forward vectors
         forward_velocity_x, forward_velocity_y = self.load_velocities(self.forward_velocity_filelist, frame_no)
@@ -410,7 +409,6 @@ class OpFlowLab:
         target_properties = object_matching.remove_items(target_properties, selected_target_mask)
 
         # Round 3
-
         magnitude = np.sqrt(np.array(matched_flow_x) ** 2 + np.array(matched_flow_y) ** 2)
         mean_magnitude = np.nanmean(magnitude)
         std_magnitude = np.nanstd(magnitude)
@@ -500,19 +498,19 @@ class OpFlowLab:
 
         values = np.c_[flow_y_cen, flow_x_cen]
 
-        interpolated_flow_x, interpolated_flow_y = interpolation.inverse_distance_weighting(pts.astype(np.int),
-                                                                                            distances, indices,
-                                                                                            values,
-                                                                                            matched_flow_x,
-                                                                                            matched_flow_y)
+        matched_flow_x, matched_flow_y = interpolation.inverse_distance_weighting(pts.astype(np.int),
+                                                                                  distances, indices,
+                                                                                  values,
+                                                                                  matched_flow_x,
+                                                                                  matched_flow_y)
 
         # x component
-        utils.save_output(interpolated_flow_x,
+        utils.save_output(matched_flow_x,
                           os.path.join(flowx_output_directory, 'flowx_{:03d}_float16'.format(frame_no + 1)),
                           save_velocity_as_tif=save_velocity_as_tif, save_dtype=save_dtype)
 
         # y component
-        utils.save_output(interpolated_flow_y,
+        utils.save_output(matched_flow_y,
                           os.path.join(flowy_output_directory, 'flowy_{:03d}_float16'.format(frame_no + 1)),
                           save_velocity_as_tif=save_velocity_as_tif, save_dtype=save_dtype)
 
@@ -592,13 +590,13 @@ class OpFlowLab:
             img_frame = None
             if alpha != 0:
                 img_frame = self.image[frame_no]
-            tracer_pos, _ = self.flowPath_iteration(frame_no, tracer_pos, all_pos_index, x_array_shape,
-                                                    flow_trace_folder,
-                                                    alpha=alpha,
-                                                    img_frame=img_frame,
-                                                    frames=frames,
-                                                    spacing=spacing,
-                                                    )
+            tracer_pos, _, _ = self.flowPath_iteration(frame_no, tracer_pos, all_pos_index, x_array_shape,
+                                                       flow_trace_folder,
+                                                       alpha=alpha,
+                                                       img_frame=img_frame,
+                                                       frames=frames,
+                                                       spacing=spacing,
+                                                       )
 
     def flowPath_initialization(self,
                                 frames=10,
@@ -655,6 +653,10 @@ class OpFlowLab:
                            img_frame=None,
                            frames=10,
                            spacing=8,
+                           colormap=None,
+                           colormap_min=None,
+                           colormap_max=None,
+                           metric_type="angle",
                            ):
         """
         Calculates and outputs vector traces for a single iteration
@@ -720,8 +722,21 @@ class OpFlowLab:
             y_diff = np.nanmean(draw_tracer_pos[-2::-1, :, 0] - draw_tracer_pos[-1:0:-1, :, 0], axis=0)
 
         # obtain colors for drawing
-        metric = np.arctan2(y_diff, -x_diff, where=~(np.isnan(x_diff) | np.isnan(y_diff))) * 180 / np.pi
-        colors = colormap_functions.get_colors(metric, plt.cm.twilight, vmin=-180, vmax=180)
+        _colormap = colormap_functions.parse_colormap(colormap)
+        colormap_type = "bar"
+        if metric_type.lower() == "angle":
+            metric = np.arctan2(y_diff, -x_diff, where=~(np.isnan(x_diff) | np.isnan(y_diff))) * 180 / np.pi
+            colormap_min = -180
+            colormap_max = 180
+            colormap_type = "wheel"
+        elif metric_type.lower() == "speed":
+            metric = np.sqrt(y_diff**2 + x_diff**2, where=~(np.isnan(x_diff) | np.isnan(y_diff)))
+        elif metric_type.lower() == "vx":
+            metric = np.where(~(np.isnan(x_diff) | np.isnan(y_diff)), x_diff, 0)
+        elif metric_type.lower() == "vy":
+            metric = np.where(~(np.isnan(x_diff) | np.isnan(y_diff)), y_diff, 0)
+
+        colors, colormap_min, colormap_max = colormap_functions.get_colors(metric, _colormap, colormap_min=colormap_min, colormap_max=colormap_max)
 
         if alpha != 0:
             output_image = skimage.color.gray2rgb(alpha * utils.imadjust(img_frame))
@@ -739,7 +754,15 @@ class OpFlowLab:
         skimage.io.imsave(os.path.join(flow_trace_folder, 'output_{:03d}.tif'.format(frame_no)), output_image,
                           check_contrast=False)
 
-        return tracer_pos, output_image
+        if colormap_type == "wheel":
+            colormap_vis = colormap_functions.save_colorwheel(_colormap,
+                                                              filename=os.path.join(flow_trace_folder, "colorwheel_{:03d}.png".format(frame_no)),)
+        elif colormap_type == "bar":
+            colormap_vis = colormap_functions.save_colorbar(_colormap,
+                                                            filename=os.path.join(flow_trace_folder, "colorbar_{:03d}.png".format(frame_no)),
+                                                            colormap_min=colormap_min, colormap_max=colormap_max, )
+
+        return tracer_pos, output_image, colormap_vis
 
     def flowTracer_function(self,
                             output_folder="FlowTracer",
@@ -873,11 +896,12 @@ class OpFlowLab:
                                tracer_folder,
                                frame_no, tracer_centroids, color,
                                ):
-        first_image = skimage.color.gray2rgb(self.image[frame_no, ...]) / 255
+        first_image = skimage.util.img_as_float32(utils.imadjust(self.image[frame_no, ...]))
+        first_image = skimage.color.gray2rgb(first_image)
         output_image = tracer_functions.draw_tracer(first_image, tracer_centroids, color)
 
         skimage.io.imsave(os.path.join(tracer_folder, 'output_{:03d}.tif'.format(frame_no)),
-                          (output_image * 255).astype(np.uint8))
+                          skimage.util.img_as_ubyte(output_image))
 
         return output_image
 
@@ -921,11 +945,12 @@ class OpFlowLab:
         draw_colors = color[draw_pos]
         draw_tracer_pos = np.rint(draw_tracer_pos)
 
-        output_image = tracer_functions.draw_tracer(skimage.color.gray2rgb(self.image[frame_no + 1, ...]) / 255,
+        output_image = skimage.util.img_as_float32(utils.imadjust(self.image[frame_no+1, ...]))
+        output_image = tracer_functions.draw_tracer(skimage.color.gray2rgb(output_image),
                                                     draw_tracer_pos, draw_colors)
 
         skimage.io.imsave(os.path.join(tracer_folder, 'output_{:03d}.tif'.format(frame_no + 1)),
-                          (output_image * 255).astype(np.uint8))
+                          skimage.util.img_as_ubyte(output_image))
 
         return tracer_centroids, output_image
 
@@ -935,7 +960,7 @@ class OpFlowLab:
                                  file_string=None,
                                  smoothing=None,
                                  colormap=None,
-                                 colormap_min=-1.5, colormap_max=1.5,
+                                 colormap_min=None, colormap_max=None,
                                  ):
         """
         Calculates gradients and output the various flow derivatives for a single iteration
@@ -989,19 +1014,26 @@ class OpFlowLab:
         if _colormap is None:
             skimage.io.imsave(os.path.join(output_folder, file_string.format(frame_no)), stat,
                               check_contrast=False)
+            colormap_vis = None
         else:
-            output_image = colormap_functions.get_colors(stat, _colormap, vmin=colormap_min, vmax=colormap_max)
+            output_image, colormap_min, colormap_max = colormap_functions.get_colors(stat, _colormap, colormap_min=colormap_min, colormap_max=colormap_max)
             output_image = output_image[..., :3]  # remove alpha layer
             output_image = skimage.util.img_as_ubyte(output_image)
             skimage.io.imsave(os.path.join(output_folder, file_string.format(frame_no)), output_image,
                               check_contrast=False)
 
-        return stat
+            stat = output_image
+            colormap_vis = colormap_functions.save_colorbar(_colormap,
+                                             filename = os.path.join(output_folder, "colorbar_{:03d}.png".format(frame_no)),
+                                             colormap_min=colormap_min, colormap_max=colormap_max,)
+
+        return stat, colormap_vis
 
     def flowCurl_function(self,
                           output_folder="curl",
                           smoothing=None,
-                          vmin=-1.5, vmax=1.5,
+                          colormap=None,
+                          colormap_min=None, colormap_max=None,
                           ):
         """
         Calculates gradients and output the curl values for an image stack
@@ -1012,9 +1044,9 @@ class OpFlowLab:
             Folder name that will be created in the main directory to store flow derivative images
         smoothing : int or Bool
             Sigma value for gaussian smoothing
-        vmin : float
+        colormap_min : float
             Minimum value that will be used for colormap normalization
-        vmax : float
+        colormap_max : float
             Maximum value that will be used for colormap normalization
 
         Returns
@@ -1028,18 +1060,21 @@ class OpFlowLab:
         os.makedirs(curl_folder, exist_ok=True)
         curl_string = 'curl_{:03d}.tif'
 
+        colormap = colormap_functions.parse_colormap(colormap)
+
         curl_stat_matrix = np.array([[-1, 0], [0, 1]])
         for img_no in tqdm(range(self.length - 1)):
             self.flowDerivative_iteration(curl_folder, img_no, curl_stat_matrix, curl_string,
                                           smoothing=smoothing,
-                                          colormap=plt.cm.coolwarm,
-                                          colormap_min=vmin, colormap_max=vmax,
+                                          colormap=colormap,
+                                          colormap_min=colormap_min, colormap_max=colormap_max,
                                           )
 
     def flowDivergence_function(self,
                                 output_folder="divergence",
                                 smoothing=None,
-                                vmin=-1.5, vmax=1.5,
+                                colormap=None,
+                                colormap_min=None, colormap_max=None,
                                 ):
         """
         Calculates gradients and output the divergence values for an image stack
@@ -1050,9 +1085,9 @@ class OpFlowLab:
             Folder name that will be created in the main directory to store flow derivative images
         smoothing : int or Bool
             Sigma value for gaussian smoothing
-        vmin : float
+        colormap_min : float
             Minimum value that will be used for colormap normalization
-        vmax : float
+        colormap_max : float
             Maximum value that will be used for colormap normalization
 
         Returns
@@ -1066,18 +1101,21 @@ class OpFlowLab:
         os.makedirs(divergence_folder, exist_ok=True)
         divergence_string = 'divergence_{:03d}.tif'
 
+        colormap = colormap_functions.parse_colormap(colormap)
+
         divergence_stat_matrix = np.array([[0, 1], [1, 0]])
         for img_no in tqdm(range(self.length - 1)):
             self.flowDerivative_iteration(divergence_folder, img_no, divergence_stat_matrix, divergence_string,
                                           smoothing=smoothing,
-                                          colormap=plt.cm.coolwarm,
-                                          colormap_min=vmin, colormap_max=vmax,
+                                          colormap=colormap,
+                                          colormap_min=colormap_min, colormap_max=colormap_max,
                                           )
 
     def flowShear_function(self,
                            output_folder="shear",
                            smoothing=None,
-                           vmin=-1.5, vmax=1.5,
+                           colormap=None,
+                           colormap_min=None, colormap_max=None,
                            ):
         """
         Calculates gradients and output the simple shear values for an image stack
@@ -1088,9 +1126,9 @@ class OpFlowLab:
             Folder name that will be created in the main directory to store flow derivative images
         smoothing : int or Bool
             Sigma value for gaussian smoothing
-        vmin : float
+        colormap_min : float
             Minimum value that will be used for colormap normalization
-        vmax : float
+        colormap_max : float
             Maximum value that will be used for colormap normalization
 
         Returns
@@ -1104,10 +1142,12 @@ class OpFlowLab:
         os.makedirs(shear_folder, exist_ok=True)
         shear_string = 'shear_{:03d}.tif'
 
+        colormap = colormap_functions.parse_colormap(colormap)
+
         shear_stat_matrix = np.array([[1, 0], [0, 1]])
         for img_no in tqdm(range(self.length - 1)):
             self.flowDerivative_iteration(shear_folder, img_no, shear_stat_matrix, shear_string,
                                           smoothing=smoothing,
-                                          colormap=plt.cm.coolwarm,
-                                          colormap_min=vmin, colormap_max=vmax,
-                                          )
\ No newline at end of file
+                                          colormap=colormap,
+                                          colormap_min=colormap_min, colormap_max=colormap_max,
+                                          )
