@@ -175,7 +175,6 @@ class tabWidget(QWidget):
     @pyqtSlot(str, str)
     def updateConsoleSignal(self, text, signal_type):
         self.console.moveCursor(QTextCursor.End)
-        tf = self.console.currentCharFormat()
         if signal_type == "error":
             color = "191, 43, 86"
         elif signal_type == "warn":
@@ -226,7 +225,7 @@ class tabWidget(QWidget):
 
     def onConsoleButtonClicked(self):
         filepath, _ = QFileDialog.getSaveFileName(self, "Choose folder to save console log", "", filter="Text file (*.txt);;")
-        print("Console log saved to {}".format(filepath))
+        print("Console log saved to \"{}\"".format(filepath))
         with open(filepath, 'w') as file:
             file.write(str(self.console.toPlainText()))
 
